@@ -1,4 +1,4 @@
-# for pickle
+﻿# for pickle
 from . import pickle_PyQt5_PySide2
 
 # for serializejson
