@@ -1,11 +1,23 @@
-try:
+﻿try:
     import qtpy
 
     API = qtpy.API_NAME
     from qtpy import QtGui, QtWidgets, QtCore
+
+    if not hasattr(QtCore, "SignalInstance"):
+        QtCore.SignalInstance = QtCore.pyqtBoundSignal  # qtpy < ?
+
+
 except ModuleNotFoundError:
     try:
         from PyQt5 import QtGui, QtWidgets, QtCore
+
+        QtCore.Signal = QtCore.pyqtSignal
+        QtCore.SignalInstance = QtCore.pyqtBoundSignal
+        QtCore.Slot = QtCore.pyqtSlot
+        QtCore.Property = QtCore.pyqtProperty
+
+        # from PyQt5.QtCore import QT_VERSION_STR as __version__
     except ModuleNotFoundError:
         try:
             from PySide2 import QtGui, QtWidgets, QtCore
@@ -29,9 +41,10 @@ if API:
             constructors,
             const,
             consts,
+            class_str_from_class,
         )
         from parse import parse
-    except:
+    except ModuleNotFoundError:
         from serializejson.tools import (
             setters,
             property_types,
@@ -42,8 +55,11 @@ if API:
             constructors,
             const,
             consts,
+            class_str_from_class,
         )
-    from SmartFramework.image.image_conversion import QImage_to_bytes_width_height_format
+    from SmartFramework.image.image_conversion import (
+        QImage_to_bytes_width_height_format,
+    )
     import ctypes
 
     import sys
@@ -51,6 +67,7 @@ if API:
     property_types.add(QtCore.Property)
 
     if API.startswith("PyQt"):
+        remove_types = None
 
         def serializejson_reducableQt(self):
             tuple_reduce = self.__reduce__()
@@ -58,6 +75,7 @@ if API:
             return type_str(self), initargs, None
 
     else:
+        remove_types = [QtCore.SignalInstance]
 
         def serializejson_reducableQt(self):
             tuple_reduce = self.__reduce__()
@@ -183,6 +201,7 @@ if API:
             "qtpy.QtGui.QPalette",
             "qtpy.QtGui.QCursor",
             "qtpy.QtCore.Qt.Alignment",
+            "qtpy.QtCore.Qt.AlignmentFlag",
             "qtpy.QtGui.QRegion",
             "qtpy.QtWidgets.QSizePolicy",
             "qtpy.QtCore.Qt.WindowStates",
@@ -265,28 +284,30 @@ if API:
     QtCore.QSizeF.__serializejson__ = serializejson_reducableQt
     QtCore.QTime.__serializejson__ = serializejson_reducableQt
 
+    def QOBject_gestate(self):
+        return getstate(
+            self,
+            split_dict_slots=False,
+            keep=None,
+            add=None,
+            remove=["parent"],
+            filter_=True,
+            properties=False,
+            getters=False,
+            sort_keys=True,
+            remove_default_values=False,
+            remove_types=remove_types,
+        )
+
+    QtCore.QObject.__getstate__ = QOBject_gestate
+
     def serializejson_QOBject(self):
         parent = self.parent()
         if parent is not None:
             init = {"parent": parent}
         else:
             init = tuple()
-        return (
-            type_str(self),
-            init,
-            getstate(
-                self,
-                split_dict_slots=False,
-                keep=None,
-                add=None,
-                remove=["parent"],
-                filter_=True,
-                properties=True,
-                getters=True,
-                sort_keys=True,
-                remove_default_values=False,
-            ),
-        )
+        return (type_str(self), init, self.__getstate__())
 
     QtCore.QObject.__serializejson__ = serializejson_QOBject
 
@@ -301,8 +322,8 @@ if API:
                 add=None,
                 remove=["parent"],
                 filter_=True,
-                properties=True,
-                getters=True,
+                properties=False,
+                getters=False,
                 sort_keys=True,
                 remove_default_values=False,
             ),
@@ -312,12 +333,15 @@ if API:
     QtCore.QLocale.__serializejson__ = serializejson_no_parent
     QtGui.QRegion.__serializejson__ = serializejson_no_parent
     QtWidgets.QSizePolicy.__serializejson__ = serializejson_no_parent
-    QtCore.Qt.WindowFlags.__serializejson__ = serializejson_no_parent
-    QtCore.Qt.WindowStates.__serializejson__ = serializejson_no_parent
-    QtCore.Qt.Alignment.__serializejson__ = serializejson_no_parent
-    QtWidgets.QMainWindow.DockOptions.__serializejson__ = serializejson_no_parent
-    QtCore.Qt.TextInteractionFlags.__serializejson__ = serializejson_no_parent
-    QtCore.QLocale.NumberOptions.__serializejson__ = serializejson_no_parent
+    QtCore.Qt.AlignmentFlag.__serializejson__ = serializejson_no_parent
+
+    if API != "PyQt6":
+        QtCore.Qt.WindowFlags.__serializejson__ = serializejson_no_parent
+        QtCore.Qt.WindowStates.__serializejson__ = serializejson_no_parent
+        QtCore.Qt.Alignment.__serializejson__ = serializejson_no_parent
+        QtWidgets.QMainWindow.DockOptions.__serializejson__ = serializejson_no_parent
+        QtCore.Qt.TextInteractionFlags.__serializejson__ = serializejson_no_parent
+        QtCore.QLocale.NumberOptions.__serializejson__ = serializejson_no_parent
 
     def serializejson_positionnal_parent(self):
         return (
@@ -330,8 +354,8 @@ if API:
                 add=None,
                 remove=["parent"],
                 filter_=True,
-                properties=True,
-                getters=True,
+                properties=False,
+                getters=False,
                 sort_keys=True,
                 remove_default_values=False,
             ),
@@ -348,13 +372,14 @@ if API:
             add=None,
             remove=["parent"],
             filter_=True,
-            properties=True,
-            getters=True,
+            properties=False,
+            getters=False,
             sort_keys=True,
             remove_default_values=False,
         )
         state["widgets"] = widgets
-        return type_str(self), (self.parent(),), state
+        # return type_str(self), (self.parent(),), None #state
+        return type_str(self), None, state  #
 
     def QLayout_setWidgets(self, widgets):
         for widget in widgets:
@@ -367,7 +392,9 @@ if API:
         widgets = []
         for i in range(self.count()):
             item = self.itemAt(i)
-            widgets.append([item.widget()] + list(self.getItemPosition(i)) + [item.alignment()])
+            widgets.append(
+                [item.widget()] + list(self.getItemPosition(i)) + [item.alignment()]
+            )
         state = getstate(
             self,
             split_dict_slots=False,
@@ -375,13 +402,13 @@ if API:
             add=None,
             remove=["parent"],
             filter_=True,
-            properties=True,
-            getters=True,
+            properties=False,
+            getters=False,
             sort_keys=True,
             remove_default_values=False,
         )
         state["widgets"] = widgets
-        return type_str(self), (self.parent(),), state
+        return type_str(self), (self.parent(),), None  # state
 
     def QGridLayout_setWidgets(self, widgets):
         for widget in widgets:
@@ -404,10 +431,11 @@ if API:
                 add=None,
                 remove=remove,
                 filter_=True,
-                properties=True,
-                getters=True,
+                properties=False,
+                getters=False,
                 sort_keys=True,
                 remove_default_values=False,
+                remove_types=remove_types,
             ),
         )
 
@@ -421,7 +449,9 @@ if API:
     def serializejson_InputMethodHints(self):
         return type_str(self), (int(self),)
 
-    QtCore.Qt.InputMethodHints.__serializejson__ = serializejson_InputMethodHints
+    if API != "PyQt6":
+        QtCore.Qt.InputMethodHints.__serializejson__ = serializejson_InputMethodHints
+
     # ---  QT GUI ------------------------------------------------------------
 
     QtGui.QPalette.__serializejson__ = serializejson_no_parent
@@ -430,7 +460,11 @@ if API:
     def serializejson_QPen(self):
         # ne sauvegarde pas le brush si n'apporte rien de plus que la couleure
         brush = self.brush()
-        if brush.style() == 1 and not brush.texture().size().width() and brush.transform() == QtGui.QTransform():
+        if (
+            brush.style() == 1
+            and not brush.texture().size().width()
+            and brush.transform() == QtGui.QTransform()
+        ):
             remove = "brush"
         else:
             remove = None
@@ -444,7 +478,7 @@ if API:
             properties=True,
             getters=True,
             sort_keys=True,
-            remove_default_values=False,
+            remove_default_values=True,
         )
         return type_str(self), tuple(), state
 
@@ -461,8 +495,8 @@ if API:
                 add=None,
                 remove="texture",
                 filter_=True,
-                properties=True,
-                getters=True,
+                properties=False,
+                getters=False,
                 sort_keys=True,
                 remove_default_values=False,
             ),
@@ -478,10 +512,9 @@ if API:
         availableSizes = self.availableSizes()
         if availableSizes != other.availableSizes():
             return False
-        elif not availableSizes:
+        if not availableSizes:
             return True
-        else:  # meme availableSizes
-            return False  # il faudrait comparer plus en profondeur
+        return False  # il faudrait comparer plus en profondeur
 
     QtGui.QIcon.__eq__ = QIcon__eq__
 
@@ -496,8 +529,8 @@ if API:
                 add=None,
                 remove=None,
                 filter_=True,
-                properties=True,
-                getters=True,
+                properties=False,
+                getters=False,
                 extra_getters={"letterSpacingType": "letterSpacingType"},
                 sort_keys=True,
                 remove_default_values=False,
@@ -529,8 +562,8 @@ if API:
                 add=None,
                 remove=None,
                 filter_=True,
-                properties=True,
-                getters=True,
+                properties=False,
+                getters=False,
                 sort_keys=True,
                 remove_default_values=False,
             ),
@@ -558,8 +591,9 @@ if API:
         ok = self.save(buff, "PBM")
         assert ok
         data = ba.data()  # fait une copie ?
-        return f"QtGui.QBitmap.fromData", (
-            data,
+        return (
+            f"QtGui.QBitmap.fromData",
+            (data,),
         )  # rien à partir de la fonction fromData ne permet de savoir qu'elle est une methode de QImage , ni à quel module ell
 
     QtGui.QBitmap.__serializejson__ = serializejson_QBitmap
@@ -587,47 +621,74 @@ if API:
     QtGui.QKeySequence.__serializejson__ = serializejson_reducableQt
     QtGui.QTransform.__serializejson__ = serializejson_reducableQt
     QtGui.QVector3D.__serializejson__ = serializejson_reducableQt
-    QtGui.QColor.__serializejson__ = serializejson_reducableQt
+
+    if API.startswith("PyQt"):
+        QtGui.QColor.__serializejson__ = serializejson_reducableQt
+    else:  # pour PySide
+
+        def serializejson_QColor(self):
+            return (
+                type_str(self),
+                (self.red(), self.green(), self.blue(), self.alpha()),
+                None,
+            )
+
+        QtGui.QColor.__serializejson__ = serializejson_QColor
 
     # def serializejson_QWidget(self):
     #    return type_str(self), {"parent": self.parent()}, getstate(self,split_dict_slots = False, keep=None, add= None, remove = ["parent","minimumSize"], filter_= True, properties=True, getters=True, sort_keys = True, remove_default_values = False)
     # QtWidgets.QWidget.__serializejson__ = serializejson_QWidget
 
+    last_classes = (QtWidgets.QLayout, QtWidgets.QGridLayout)
+
+    def QWidget_getstate(self):
+        remove = ["parent", "cursor"]
+        if self.layout() is None:
+            remove.append("layout")
+        return getstate(
+            self,
+            split_dict_slots=False,
+            keep=None,
+            add=None,
+            remove=remove,
+            filter_=True,
+            properties=False,
+            getters=False,
+            sort_keys=True,
+            remove_default_values=False,
+            last_classes=last_classes,
+            remove_types=remove_types,
+        )
+
+    QtWidgets.QWidget.__getstate__ = QWidget_getstate
+
     def serializejson_QWidget(self):
         parent = self.parent()
         if parent is not None:
             init = {"parent": parent}
         else:
             init = tuple()
-        remove = ["parent", "cursor"]
-        if self.layout() is None:
-            remove.append("layout")
         return (
             type_str(self),
             init,
-            getstate(
-                self,
-                split_dict_slots=False,
-                keep=None,
-                add=None,
-                remove=remove,
-                filter_=True,
-                properties=True,
-                getters=True,
-                sort_keys=True,
-                remove_default_values=False,
-            ),
+            self.__getstate__(),
         )
 
     QtWidgets.QWidget.__serializejson__ = serializejson_QWidget
 
+    def QSpinBox_getstate(self):
+        return {"value": self.value()}
+
+    QtWidgets.QDoubleSpinBox.__getstate__ = QSpinBox_getstate
+    QtWidgets.QSpinBox.__getstate__ = QSpinBox_getstate
+
     def serializejson_QSpinBox(self):
-        state = {"value": self.value()}
-        return type_str(self), tuple(), state
+        return type_str(self), tuple(), self.__getstate__()
 
     QtWidgets.QSpinBox.__serializejson__ = serializejson_QSpinBox
     QtWidgets.QDoubleSpinBox.__serializejson__ = serializejson_QSpinBox
-    setters[QtWidgets.QSpinBox] = True  # get_properties(QtWidgets.QSpinBox) {'value' : 'setValue'}
+    # get_properties(QtWidgets.QSpinBox) {'value' : 'setValue'}
+    setters[QtWidgets.QSpinBox] = True
     setters[QtWidgets.QDoubleSpinBox] = True  # {'value' : 'setValue'}
 
     def serializejson_QCheckBox(self):
@@ -641,10 +702,11 @@ if API:
                 add=None,
                 remove=["parent", "cursor", "windowFlags"],
                 filter_=True,
-                properties=True,
-                getters=True,
+                properties=False,
+                getters=False,
                 sort_keys=True,
                 remove_default_values=False,
+                remove_types=remove_types,
             ),
         )  # ["checked"])
 
@@ -673,14 +735,17 @@ if API:
     QtWidgets.QPlainTextEdit.__serializejson__ = serializejson_QPlainTextEdit
     setters[QtWidgets.QPlainTextEdit] = {"plainText": "setPlainText"}
 
-    # allow translation PyQt5 -> qtpy and PySide2 -> qtpy if qtpy avaible
+    # vire le prefixe PyQt5. et PySide2.
     def type_str(self):
-        return f"{str(self.__class__)[len(API)+9:-2]}"
+        s = class_str_from_class(type(self))
+        if s.startswith(API):
+            return s[len(API) + 1 :]
+        return s
 
     # ENUMS -----------------
 
-    def serializejson_Enum(value):
-        return "const", (qt_const_name[(type(value), value)],)
+    def serializejson_Enum(self):
+        return "const", (qt_const_name[(type(self), self)],)
 
     qt_const_name = dict()
     qt_const_type = type(QtCore.Qt.CheckState)
@@ -688,7 +753,6 @@ if API:
     for key, value in QtCore.Qt.__dict__.items():
         type_value = type(value)
         if type_value is qt_const_type:
-            pass
             value.__serializejson__ = serializejson_Enum
         if type(type_value) is qt_const_type:
             str_ = f"QtCore.Qt.{key}"
@@ -701,46 +765,61 @@ if API:
                 for key, value in class_.__dict__.items():
                     type_value = type(value)
                     if type_value is qt_const_type:
-                        pass
                         value.__serializejson__ = serializejson_Enum
                     if type(type_value) is qt_const_type:
                         try:
                             str_ = f"{module_name}.{class_name}.{key}"
                             qt_const_name[(type(value), value)] = str_
                             consts[str_] = value
-                        except:
+                        except TypeError:
                             pass
-                        else:
-                            pass
-                            # print('ok')
     constructors["const"] = const
 
     # SERIALISATION DES CONNECTIONS -----------------------------------------------
 
-    # hack pour enregistrer les connections
-    old_connect = QtCore.pyqtBoundSignal.connect
+    # avec pyside les connection sont des elements de __dict__ !!!
+    # je ne les serialize pas pour l'instant en mettant  remove_types = [QtCore.SignalInstance]
+    # mais on pourrait facilement les serializer à priori ! :)
+
+    # hack pour enregistrer les connections dans PyQt6
+
+    old_connect = QtCore.SignalInstance.connect
 
     def new_connect(signal, slot, save=None):
-        value = old_connect(signal, slot)
+        return_value = old_connect(signal, slot)
         if save is True or (
             save is None and sys._getframe(1).f_code.co_name != "__init__"
         ):  # sauve la connection si elle a été crée en dehors d'un __init__
-            signal_object, signal_name, slot_object, slot_name, signature = connection_infos(signal, slot)
+            (
+                signal_object,
+                signal_name,
+                slot_object,
+                slot_name,
+                signature,
+            ) = connection_infos(signal, slot)
             signal_parents = get_parents(signal_object)
             slot_parents = get_parents(slot_object)
             for commun_parent in signal_parents:
                 if commun_parent in slot_parents:
                     break
             else:
-                print("No Commun Qt Parent for ", signal_object, signal_parents, slot_object, slot_parents)
+                print(
+                    "No Commun Qt Parent for ",
+                    signal_object,
+                    signal_parents,
+                    slot_object,
+                    slot_parents,
+                )
             if "~connections" not in commun_parent.__dict__:
                 commun_parent.__dict__["~connections"] = []
             commun_parent.__dict__["~connections"].append(
-                Connection(signal_object, signal_name, slot_object, slot_name, signature)
+                Connection(
+                    signal_object, signal_name, slot_object, slot_name, signature
+                )
             )
-        return value
+        return return_value
 
-    # QtCore.pyqtBoundSignal.connect = new_connect
+    # QtCore.SignalInstance.connect = new_connect
 
     class Connection:
         def __init__(self, signal_object, signal, slot_object, slot, signature=None):
@@ -759,7 +838,7 @@ if API:
                 "Connection",
                 None,
                 {
-                    "signal": Reference(self.signal_object, signal_name_sig),  # #Reference(signal),
+                    "signal": Reference(self.signal_object, signal_name_sig),
                     "slot": Reference(
                         self.slot_object, "." + self.slot_name
                     ),  # Reference([self.slot_object,self.slot_name]),
@@ -771,9 +850,13 @@ if API:
             signal = state["signal"]
             slot = state["slot"]
             signal.connect(slot, save=False)
-            self.signal_object, self.signal_name, self.slot_object, self.slot_name, self.signature = connection_infos(
-                signal, slot
-            )
+            (
+                self.signal_object,
+                self.signal_name,
+                self.slot_object,
+                self.slot_name,
+                self.signature,
+            ) = connection_infos(signal, slot)
 
     constructors["Connection"] = Connection
     signature_str_from_qt = {
@@ -784,18 +867,30 @@ if API:
         "QString": "str",
         "PyQt_PyObject": "object",
     }
-    signature_from_srt = {"bool": bool, "int": int, "float": float, "str": str, "object": object}
+    signature_from_srt = {
+        "bool": bool,
+        "int": int,
+        "float": float,
+        "str": str,
+        "object": object,
+    }
 
     def connection_infos(signal, slot):
         signal_str = signal.__str__()
-        signal_name, _, hex_id = parse("<bound PYQT_SIGNAL {} of {} object at {}>", signal_str).fixed
+        signal_name, _, hex_id = parse(
+            "<bound PYQT_SIGNAL {} of {} object at {}>", signal_str
+        ).fixed
         signal_object = ctypes.cast(int(hex_id, 16), ctypes.py_object).value
         signal_str = signal.signal
-        signature = signature_str_from_qt.get(signal_str[signal_str.rfind("(") + 1 : -1], "object")
+        signature = signature_str_from_qt.get(
+            signal_str[signal_str.rfind("(") + 1 : -1], "object"
+        )
 
-        if isinstance(slot, QtCore.pyqtBoundSignal):
+        if isinstance(slot, QtCore.SignalInstance):
             slot_str = slot.__str__()
-            slot_name, _, hex_id = parse("<bound PYQT_SIGNAL {} of {} object at {}>", slot_str).fixed
+            slot_name, _, hex_id = parse(
+                "<bound PYQT_SIGNAL {} of {} object at {}>", slot_str
+            ).fixed
             slot_object = ctypes.cast(int(hex_id, 16), ctypes.py_object).value
         else:
             slot_object = slot.__self__
