@@ -1,4 +1,4 @@
-try:
+﻿try:
     from SmartFramework.serialize.tools import serializejson_, authorized_classes
     from SmartFramework.serialize import serialize_parameters
 except:
