@@ -1,4 +1,4 @@
-# serialisation de tableaux numpy -------------
+﻿# serialisation de tableaux numpy -------------
 
 # numpy_array_dumped_base64  = True     # dit si doit utiliser l'encodage en base64 pour l'encodage de tableau numpy. Ce paramètre n'est modifiable que directement ici.
 # numpy_array_readable_max_size = 0    # force tout de même à  encoder les tableaux de taille <= numpy_array_readable_max_size, sous forme de liste lisible même si on utilise numpy_array_dumped_base64 = True ( et donc d'utiliser numpy.array au lieu du construcuteur partir de bytes comme numpy.ndarray (pour tableau de dim > 2 si use_numpyB64 == False ), numpyB64 ou autre), paramètre directement ici , ou en paramètre de SerializeInterface et SerializePresetUI
