@@ -1,4 +1,4 @@
-from . import serialize_parameters
+﻿from . import serialize_parameters
 from SmartFramework.string.encodings import ascii_printables
 from SmartFramework.tools.dictionaries import sorted_dict, sorted_filtered  # ,filtered
 from SmartFramework.tools.objects import (
@@ -29,15 +29,21 @@ not_memorized_types = (int, float)
 # --- PLUGINS API -------------------------------
 
 # encoding -------------
-dispatch_table  # pickle plugins (used by serializejson if not serializejson plugin or methode)
+# pickle plugins (used by serializejson if not serializejson plugin or methode)
+dispatch_table
 serializejson_ = {}  # serializejson plugins
-serializejson_builtins = {}  # serializejson plugins actives even if strict_pickle is True
-encoder_parameters = {}  # encoder extra parameters for plugins, with their default value
+# serializejson plugins actives even if strict_pickle is True
+serializejson_builtins = {}
+# encoder extra parameters for plugins, with their default value
+encoder_parameters = {}
+
 getters = (
     {}
 )  # getters for dumped classes. keys are string corresponding to the qualified name, values are True (for automatic getters detection) or dictionnary of {"attribut" : "getAttribut" }
 property_types = {property}  # property types
-blosc_compressions = {(name if name == "blosclz" else "blosc_" + name): name for name in blosc.cnames}
+blosc_compressions = {
+    (name if name == "blosclz" else "blosc_" + name): name for name in blosc.cnames
+}
 # encoding & decoding --------------
 
 properties = (
@@ -55,6 +61,8 @@ decoder_parameters = {}  # decoder extra parameters for plugins with their defau
 consts = {}  # dictionnary associating const string to const values
 
 # @profile
+
+
 def getstate(
     self,
     *,
@@ -62,14 +70,16 @@ def getstate(
     keep=None,
     add=None,
     remove=None,
+    remove_types=None,
     filter_="_",
     properties=False,
     getters=False,
     extra_getters=None,
     sort_keys=True,
+    lasts=None,
+    last_classes=None,
     remove_default_values=False,
     default_values=None,
-    lasts=None,
 ):
     """
     Generic __gestate__ method to retrieve the state of an object .
@@ -79,17 +89,21 @@ def getstate(
         keep :   names of attributes/properties/getters to keep (and order if sort_keys is False)
         add :    names of attributes to add even if should be filtered by the filter
         remove:  names of attributes to remove even if not filtered by the filter
+        remove_types:  types (not strings) of attributes to remove even if not filtered by the filter
+
+
         filter\_: (bool or str) filter attributes starting with the given string. ("_" by default)
 
         properties :
-                False: (default) no properties are saved
-                True : all properties (or only thus in keep) are saved
-                list or tuple : names of properties to save
+                Whether properties will be saved.
+                - `False`: (default) no properties are saved
+                - `True` : all properties (or only thus in keep) are saved
+                - `list/tuple` : names of properties to save
         getters :
-                False :(default) None getters are saved
-                True : all getters (or only thus in keep) are saved. (getters are guessed with introspection)
-                dict : dictionnary of "attribut": "getter". ex: {"a":"getA","b":"getB"}
-                this option allow the finest control and is faster because getters are not guessed from instropection.
+                Whether values from getters will be saved.
+                - `False` :(default) None getters are saved
+                - `True` : all getters (or only thus in keep) are saved. (getters are guessed with introspection)
+                - `dict` : dictionnary of "attribut": "getter". ex: {"a":"getA","b":"getB", ("c","d") : "getCD"} this option allow the finest control and is faster because getters are not guessed from introspection. With tuple as key in this dict, you can retrieve several attributes values from one getter.
 
         extra_getters
                 dictionnary of extra getters.    ex: {"c":"retrive_c"}
@@ -99,6 +113,14 @@ def getstate(
                 whether sort the state alphabeticaly.
                 Be careful, if False the restoration of the object attributes may be in an arbitrary order.
 
+
+        lasts : names of attributes/properties/getters to put at last  (and order if sort_keys is False)
+        last_classes (set):
+                set of classes to put at the end (even if sort_keys is True)
+                Useful for classes referencing other objets, when you want to be sure that the referenced obejcts has already been serialized sooner in order to serialize only reférences.
+                (ex : Qt Layouts)
+
+
         remove_default_values :(False, True)
                 whether attribut/properties/getter with same value as default value will be
                 removed for lighter and more readable serialized files.
@@ -106,9 +128,8 @@ def getstate(
                 use add = "attribut_name" parameter
 
         default_values : (None or dict)
-                dict : dict of {"attribut":"attribut_default_value",...;}
-                None :  serializejson will create an new instance of the object's
-                class, calling __init__() without any argument to know the default values.
+                - `dict` : dict of {"attribut":"attribut_default_value",...;}
+                - `None` :  serializejson will create an new instance of the object's class, calling __init__() without any argument to know the default values.
     """
     #            ,...,("x","y",None):"getXYZ"}
     #        With tuple as key ou can retrieve several attributes values from one getter.
@@ -126,7 +147,12 @@ def getstate(
     has_slots = hasattr(self, "__slots__")
 
     if (properties is True) or (getters is True):
-        slots, class_properties, class_getters, _ = slots_properties_getters_setters_from_class(type(self))
+        (
+            slots,
+            class_properties,
+            class_getters,
+            _,
+        ) = slots_properties_getters_setters_from_class(type(self))
         if properties is True:
             properties = class_properties
         if getters is True:
@@ -138,7 +164,8 @@ def getstate(
             getters = {}
     if extra_getters:
         getters.update(extra_getters)
-    if split_dict_slots and (has_slots or properties or getters):  # ATTENTION PAS LE CAS SI PAS DE __SLOT__ ?
+    if split_dict_slots and (has_slots or properties or getters):
+        # ATTENTION PAS LE CAS SI PAS DE __SLOT__ ?
         state_dict = dict()
         state_slots = dict()
         splited_dict_slots = True
@@ -150,7 +177,8 @@ def getstate(
         for key in keep:
             if key in getters:
                 state_slots[key] = _getattribute(getters[key])()
-            elif key in properties:  # on commenc pas voir si existe sous forme de propriété
+            elif key in properties:
+                # on commenc par voir si existe sous forme de propriété
                 state_slots[key] = _getattribute(
                     key
                 )  # le stocker dans state_slots permet à pickle de restaurer des properties dans pickle._dumps.load_build :if slotstate:for k, v in slotstate.items():setattr(inst, k, v)
@@ -174,27 +202,36 @@ def getstate(
         if properties:
             for key in properties:
                 if key not in remove:
-                    state_slots[key] = _getattribute(
-                        key
-                    )  # le stocker dans state_slots permet à pickle de restaurer des properties dans pickle._dumps.load_build :if slotstate:for k, v in slotstate.items():setattr(inst, k, v)
+                    value = _getattribute(key)
+                    if remove_types and (type(value) in remove_types):
+                        continue
+                    # le stocker dans state_slots permet à pickle de restaurer des properties dans pickle._dumps.load_build :if slotstate:for k, v in slotstate.items():setattr(inst, k, v)
+                    state_slots[key] = value
         # get getters -------------
         if getters:
             for key, getter_name in getters.items():
                 if type(key) is tuple:
                     result = _getattribute(getter_name)()
                     for k, value in zip(key, result):
-                        if k is not None and k not in remove:
+                        if (
+                            k is not None
+                            and k not in remove
+                            and not (remove_types and (type(value) in remove_types))
+                        ):
                             state_slots[k] = value
                 elif key not in remove:
                     try:
-                        state_slots[key] = _getattribute(getter_name)()
+                        value = _getattribute(getter_name)()
+                        if remove_types and (type(value) in remove_types):
+                            continue
+                        state_slots[key] = value
                     except TypeError:
                         pass  # n'arrive pas à regler le pb avec inspect qui n'arrive pas à parser arguement du getter
 
         # get __slots__ attributs ----------------
         check_prop_getters = bool(state_slots)
         if has_slots:
-            if remove or filter_ or check_prop_getters:
+            if remove or filter_ or check_prop_getters or remove_types:
                 for key in slots:
                     if (
                         key not in remove
@@ -202,25 +239,38 @@ def getstate(
                         and ((key in add) or not (filter_ and key.startswith(filter_)))
                         and not (
                             check_prop_getters
-                            and (key in state_slots or (key.startswith("_") and key[1:] in state_slots))
+                            and (
+                                key in state_slots
+                                or (key.startswith("_") and key[1:] in state_slots)
+                            )
                         )
                     ):
-                        state_slots[key] = _getattribute(key)
+                        value = _getattribute(key)
+                        if remove_types and (type(value) in remove_types):
+                            continue
+                        state_slots[key] = value
             else:
                 for key in slots:
                     if hasattr(self, key):
                         state_slots[key] = _getattribute(key)
         # get __dict__ attributs -----------------
-        if remove or filter_ or check_prop_getters:
+        if remove or filter_ or check_prop_getters or remove_types:
             for key in _dict:
                 if (
                     key not in remove
                     and ((key in add) or not (filter_ and key.startswith(filter_)))
                     and not (
-                        check_prop_getters and (key in state_slots or (key.startswith("_") and key[1:] in state_slots))
+                        check_prop_getters
+                        and (
+                            key in state_slots
+                            or (key.startswith("_") and key[1:] in state_slots)
+                        )
                     )
                 ):
-                    state_dict[key] = _dict[key]
+                    value = _dict[key]
+                    if remove_types and (type(value) in remove_types):
+                        continue
+                    state_dict[key] = value
         else:
             if not state_slots or splited_dict_slots:
                 state_dict = _dict  # no copie   A REVOIR , ON SPLIT DICT ET SLOT MEME SI PAS SPLITE A LA BASE
@@ -243,6 +293,10 @@ def getstate(
 
     if sort_keys and state_dict:
         state_dict = sorted_dict(state_dict)
+    if last_classes is not None:
+        for key, value in state_dict.items():
+            if type(value) in last_classes:
+                state_dict[key] = state_dict.pop(key)
     if lasts is not None:
         for key in lasts:
             if key in state_dict:
@@ -251,7 +305,8 @@ def getstate(
         if sort_keys:
             state_dict = sorted_dict(state_dict)
         if state_dict:
-            return state_dict, state_slots  # peut planter si self n'a pas de __dict__ et  pas de __setstate__
+            # peut planter si self n'a pas de __dict__ et  pas de __setstate__
+            return state_dict, state_slots
         if state_slots:
             return None, state_slots
 
@@ -312,15 +367,15 @@ def setstate(
     """
 
     """methode qu'on utilsera dans les object"""
-    ## same as pickle
-    ## Commenté pour eviter boucle infinie
+    # same as pickle
+    # Commenté pour eviter boucle infinie
     ## setstate = getattr(self, "__setstate__", None)
-    ## if setstate is not None:
-    ##     setstate(state)
-    ##     return
-    ##    data_driven (bool):
-    ##        True : attributs are restored in state dictionnary key's order
-    ##        False : attribut are restored in the fallowin order : __slots__,properties,setters, other state dictionnary key's order
+    # if setstate is not None:
+    # setstate(state)
+    # return
+    # data_driven (bool):
+    # True : attributs are restored in state dictionnary key's order
+    # False : attribut are restored in the fallowin order : __slots__,properties,setters, other state dictionnary key's order
 
     # on a un dictionnaire ou eventuellemnet un tuple  (__dict__,__state__) si serializé avec pickle
     passed_dict_setters = isinstance(setters, dict)
@@ -332,9 +387,12 @@ def setstate(
             state.update(dict_state)
 
     if not type(state) is dict:
-        raise Exception("try to restore object to a no dictionary state and without __setstate__ method")
+        raise Exception(
+            "try to restore object to a no dictionary state and without __setstate__ method"
+        )
 
-    if restore_default_values:  # est-ce utile si pas d'update et qu'on vient de recrer l'objet  ?
+    if restore_default_values:
+        # est-ce utile si pas d'update et qu'on vient de recrer l'objet  ?
         if default_values is None:
             default_values = default_state_from_class(type(self))
         for key, default_value in default_values.items():
@@ -342,7 +400,12 @@ def setstate(
                 state[key] = default_value
 
     if setters is True or properties is True or hasattr(self, "__slots__"):
-        slots, class_properties, _, class_setters = slots_properties_getters_setters_from_class(type(self))
+        (
+            slots,
+            class_properties,
+            _,
+            class_setters,
+        ) = slots_properties_getters_setters_from_class(type(self))
         if setters is True:
             setters = class_setters
         if properties is True:
@@ -370,7 +433,8 @@ def setstate(
     if order is None:
         for attribut, value in state.items():
             if attribut in setattr_:
-                setattr(self, attribut, value)  # marche pour les attribut de __dict__, slots et properties
+                # marche pour les attribut de __dict__, slots et properties :
+                setattr(self, attribut, value)
             elif attribut in attribut_to_multi_attributs:
                 attributs = attribut_to_multi_attributs[attribut]
                 if attributs:
@@ -387,14 +451,16 @@ def setstate(
         for attribut in order:
             value = state.popitem(attribut)
             if attribut in setattr_:
-                setattr(self, attribut, value)  # marche pour les attribut de __dict__, slots et properties
+                # marche pour les attribut de __dict__, slots et properties
+                setattr(self, attribut, value)
             elif attribut in attribut_to_multi_attributs:
                 attributs = attribut_to_multi_attributs[attribut]
                 if attributs:
                     values = []
                     for k in attributs:
                         if k == attribut:
-                            values.append(value)  # attribut as already be poped , state[k] doesn't exciste anymore
+                            # attribut as already be poped , state[k] doesn't exciste anymore
+                            values.append(value)
                         else:
                             values.append(state[k])
                         attribut_to_multi_attributs[attribut] = False
@@ -404,7 +470,9 @@ def setstate(
             else:
                 self.__dict__[intern(attribut)] = value
         if state:
-            raise Exception(f"{list(state.keys())} are not in __setstate__ order parameter")
+            raise Exception(
+                f"{list(state.keys())} are not in __setstate__ order parameter"
+            )
     # else :
     #    sentinel = []
     #    for key in slots :
@@ -472,6 +540,36 @@ def module_str_from_class_str(class_str):
     # print(class_str)
 
 
+@cached_one_arg_func
+def import_str_from_class_str(class_str):
+    if class_str in constructors and class_str.find(".") == -1:
+        # permet de gerer le cas numpyB64
+        class_ = constructors[class_str]
+        if class_.__name__ == class_str:
+            return f"from {class_.__module__} import {class_str}"
+        else:
+            return f"from {class_.__module__} import {class_.__name__} as {class_str}"
+    else:
+        module_str = class_str
+        stop = module_str.rfind(".")
+        while stop != -1:
+            module_str = module_str[:stop]
+            if module_str in sys.modules:
+                true_module_str = sys.modules[module_str].__name__
+                if module_str == true_module_str:
+                    return f"import {module_str}"
+                else:
+                    parent_module, sub_module = true_module_str.rsplit(".", 1)
+                    if module_str == sub_module:
+                        return f"from {parent_module} import {module_str}"
+                    else:
+                        return (
+                            f"from {parent_module} import {sub_module} as {module_str}"
+                        )
+            stop = module_str.rfind(".")
+        # print(class_str)
+
+
 @cached_one_arg_func
 def class_str_from_class(class_):
     module = class_.__module__
@@ -496,7 +594,12 @@ def class_str_from_class(class_):
 def default_state_from_class(class_):
     obj = class_()
     return getstate(
-        obj, split_dict_slots=False, properties=True, getters=True, sort_keys=False, remove_default_values=False
+        obj,
+        split_dict_slots=False,
+        properties=True,
+        getters=True,
+        sort_keys=False,
+        remove_default_values=False,
     )
 
 
@@ -518,11 +621,13 @@ def slots_properties_getters_setters_from_class(class_):
         for key, value in base_class.__dict__.items():
             # print(key)
             if isinstance(value, property_types_):
-                if hasattr(value, "__set__") and hasattr(value, "__get__"):
-                    # if inspect.isdatadescriptor(value):
-                    properties.append(key)
+                # if hasattr(value, "__set__") and hasattr(value, "__get__"):
+                # if inspect.isdatadescriptor(value):
+                properties.append(key)
             elif (
-                key.startswith("set") and len(key) > 3 and ismethod_methoddescriptor_or_function(value)
+                key.startswith("set")
+                and len(key) > 3
+                and ismethod_methoddescriptor_or_function(value)
             ):  # and callable(value):
                 c = key[3]
                 if c == "_":
@@ -538,8 +643,11 @@ def slots_properties_getters_setters_from_class(class_):
                     for getter_name in (attribut_name, "g" + key[1:], "is" + key[3:]):
                         getter_methode = getattr(base_class, getter_name, None)
                         if getter_methode is not None:
-                            if getter_methode not in getters and ismethod_methoddescriptor_or_function(
-                                getter_methode
+                            if (
+                                getter_methode not in getters
+                                and ismethod_methoddescriptor_or_function(
+                                    getter_methode
+                                )
                             ):  # and callable(getter_methode):
                                 getters[attribut_name] = getter_name
                                 break
@@ -632,17 +740,26 @@ def tuple_from_reduce(func, args, state=None, listitems=None, dictitems=None, ob
                 if not hasattr(class_, "__new__"):
                     raise PicklingError("args[0] from __newobj__ args has no __new__")
                 if obj is not None and class_ is not obj.__class__:
-                    raise PicklingError("args[0] from __newobj__ args has the wrong class")
+                    raise PicklingError(
+                        "args[0] from __newobj__ args has the wrong class"
+                    )
             if len(args) > 1:
-                newArgs = args[1:]  # le reduce n'a pas été réimplémenté , on doit pas utiliser d'initArgs
+                # le reduce n'a pas été réimplémenté , on doit pas utiliser d'initArgs
+                newArgs = args[1:]
         else:  # func is __newobj_ex__
             class_, new_largs, new_kwargs = args
             if serialize_parameters.strict_pickle:
                 if not hasattr(class_, "__new__"):
-                    raise PicklingError("args[0] from {} args has no __new__".format(getattr(func, "__name__", "")))
+                    raise PicklingError(
+                        "args[0] from {} args has no __new__".format(
+                            getattr(func, "__name__", "")
+                        )
+                    )
                 if obj is not None and class_ is not obj.__class__:
                     raise PicklingError(
-                        "args[0] from {} args has the wrong class".format(getattr(func, "__name__", ""))
+                        "args[0] from {} args has the wrong class".format(
+                            getattr(func, "__name__", "")
+                        )
                     )
             if new_largs:
                 # on met les new_largs dans new_kwargs
@@ -654,15 +771,17 @@ def tuple_from_reduce(func, args, state=None, listitems=None, dictitems=None, ob
 
             if type(state) is tuple:
                 # ATTENTION en vrais rien ne nous dit qu'on a pas voulu retourne un autre tuple de longeur deux, si c'est le cas on a codé
-                if not class_has_method(class_, "__getstate__") or not class_has_method(class_, "__setstate__"):
+                if not class_has_method(class_, "__getstate__") or not class_has_method(
+                    class_, "__setstate__"
+                ):
                     __dict__, state = state
                     if __dict__:
-                        state.update(__dict__)  # fusionne slots et __dict__ dans un seul dictionnaire
+                        # fusionne slots et __dict__ dans un seul dictionnaire
+                        state.update(__dict__)
 
             if type(state) is dict:
-                if not class_has_method(
-                    class_, "__getstate__"
-                ):  # on devrait mettre ce qui suit pour être exacte mais couteux pour cas qui n'arrivera jamais : and ( class_.__reduce__ is object.__reduce__) and (class_.__reduce_ex__ is object.__reduce_ex__):
+                if not class_has_method(class_, "__getstate__"):
+                    # on devrait mettre ce qui suit pour être exacte mais couteux pour cas qui n'arrivera jamais : and ( class_.__reduce__ is object.__reduce__) and (class_.__reduce_ex__ is object.__reduce_ex__):
                     # le __reduce_ex__ a déjà recupéra les slots et attributs du __dict__
 
                     # add properties and getters (need obj)
@@ -709,7 +828,7 @@ def tuple_from_reduce(func, args, state=None, listitems=None, dictitems=None, ob
                             except TypeError:
                                 pass
 
-                    # remove default values (Je l'ai viré car il faut faire appel au __init__ et donc avoir recodé le reduce() pour pouvoir se permetre d'enlever les valeures par défaut)
+                    """# remove default values (Je l'ai viré car il faut faire appel au __init__ et donc avoir recodé le reduce() pour pouvoir se permetre d'enlever les valeures par défaut)
                     ##remove_default_values = serialize_parameters.remove_default_values
                     ##if type(remove_default_values) is set :
                     ##    remove_default_values = (class_ in remove_default_values)
@@ -721,7 +840,7 @@ def tuple_from_reduce(func, args, state=None, listitems=None, dictitems=None, ob
                     ##            if state[key] == default_value:
                     ##                if state is _dict:
                     ##                    state = _dict.copy()
-                    ##               del(state[key])
+                    ##               del(state[key])"""
                     attributes_filter = serialize_parameters.attributes_filter
                     if type(attributes_filter) is set:
                         attributes_filter = class_ in attributes_filter
@@ -761,7 +880,8 @@ def reduce(obj, protocol):
         reduce_func = getattr(obj, "__reduce_ex__", None)
         if reduce_func is not None:
             # try :
-            reduced = reduce_func(protocol)  # fait appel à __gestate__() si on n'a pas réimplementé __reduce__()
+            # fait appel à __gestate__() si on n'a pas réimplementé __reduce__()
+            reduced = reduce_func(protocol)
             # except :
             #    reduced =  generic__reduce_ex__(obj)
         else:
@@ -769,10 +889,14 @@ def reduce(obj, protocol):
             if reduce_func is not None:
                 reduced = reduce_func()
             else:
-                raise PicklingError("Can't pickle %r object: %r" % (type(obj).__name__, obj))
+                raise PicklingError(
+                    "Can't pickle %r object: %r" % (type(obj).__name__, obj)
+                )
 
     if isinstance(reduced, str):
-        raise ValueError("{} reduce's methode return a string. It is not yet suported by serializejson")  # A REVOIR !!!
+        raise ValueError(
+            "{} reduce's methode return a string. It is not yet suported by serializejson"
+        )  # A REVOIR !!!
 
     if serialize_parameters.strict_pickle:
         # Check for string returned by reduce(), meaning "save as global"
@@ -784,7 +908,9 @@ def reduce(obj, protocol):
 
         # Assert that it returned an appropriately sized tuple
         if not (2 <= len(reduced) <= 5):
-            raise PicklingError("Tuple returned by %s must have " "two to five elements" % reduce_func)
+            raise PicklingError(
+                "Tuple returned by %s must have " "two to five elements" % reduce_func
+            )
 
         if not callable(reduced[0]):
             raise PicklingError(
@@ -794,9 +920,15 @@ def reduce(obj, protocol):
 
 
 def generic__reduce_ex__(self, protocol=4):
-    if hasattr(self, "__reduce_ex__") and self.__class__.__reduce_ex__ is not object.__reduce_ex__:
+    if (
+        hasattr(self, "__reduce_ex__")
+        and self.__class__.__reduce_ex__ is not object.__reduce_ex__
+    ):
         return self.__reduce_ex__(protocol)
-    if hasattr(self, "__reduce__") and self.__class__.__reduce__ is not object.__reduce__:
+    if (
+        hasattr(self, "__reduce__")
+        and self.__class__.__reduce__ is not object.__reduce__
+    ):
         return self.__reduce__()
     state, dictitems, listitems = None, None, None
     if hasattr(self, "__getstate__"):
@@ -819,7 +951,13 @@ def generic__reduce_ex__(self, protocol=4):
 
     if hasattr(self, "__getnewargs_ex__"):
         new_args, kwargs = self.__getnewargs_ex__()
-        return __newobj_ex__, (self.__class__, new_args, kwargs), state, listitems, dictitems
+        return (
+            __newobj_ex__,
+            (self.__class__, new_args, kwargs),
+            state,
+            listitems,
+            dictitems,
+        )
     elif hasattr(self, "__getnewargs__"):
         new_args = self.__getnewargs__()
         return __newobj__, (self.__class__, *new_args), state, listitems, dictitems
@@ -880,7 +1018,13 @@ def const(self):
 
 # @profile
 def instance(
-    __class__=object, __init__=None, __state__=None, __new__=None, __initArgs__=None, __items__=None, **argsSup
+    __class__=object,
+    __init__=None,
+    __state__=None,
+    __new__=None,
+    __initArgs__=None,
+    __items__=None,
+    **argsSup,
 ):
     """créer une instance d'un objet :
     instance(dictionnaire)
@@ -908,7 +1052,9 @@ def instance(
             # acceleration en allant directement charcher la class_ à partir de la string dans un dictionnaire de cash
             class_ = class_from_class_str_dict[__class__]
         except:
-            class_ = class_from_class_str_dict[__class__] = class_from_class_str(__class__)
+            class_ = class_from_class_str_dict[__class__] = class_from_class_str(
+                __class__
+            )
     else:
         if type(__class__) is dict:
             # permet de gere le cas ou on donne directement un dictionnaire en premier argument
@@ -932,7 +1078,8 @@ def instance(
             elif __new__type is dict:
                 inst = class_.__new__(class_, **__new__)
             elif __new__ is not None:
-                inst = class_.__new__(class_, __new__)  # when braces have been removed during serialization
+                # when braces have been removed during serialization
+                inst = class_.__new__(class_, __new__)
             else:
                 inst = class_.__new__(class_)  # __init__ is None
             if (
@@ -944,7 +1091,8 @@ def instance(
                 elif __init__type is dict:
                     inst.__init__(**__init__)
                 else:
-                    inst.__init__(__init__)  # when braces have been removed during serialization
+                    # when braces have been removed during serialization
+                    inst.__init__(__init__)
         elif __init__ is not None:
             __init__type = type(__init__)
             if __init__type in (list, tuple):
@@ -952,7 +1100,8 @@ def instance(
             elif __init__type is dict:
                 inst = class_(**__init__)
             else:
-                inst = class_(__init__)  # when braces have been removed during serialization
+                # when braces have been removed during serialization
+                inst = class_(__init__)
 
     if __items__:
         try:
@@ -986,7 +1135,6 @@ valid_char_for_var_name = set("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVW
 
 
 def from_name(path, accept_dict_as_object=False, **variables):
-
     """fonction qui permet d'evaluer une expression pour acceder à une valeure à partir de son nom qualifié
     fonctionne comme eval, mais securisé en acceptant juste la qualification avec "." et l'indexation avec des []
     ATTENTION cette fonction n'a pas été testée à fond,il faudrait ecrire des tests!
@@ -1094,7 +1242,11 @@ def from_name(path, accept_dict_as_object=False, **variables):
                     if element_chars:
                         key = "".join(element_chars)
                         if in_var:
-                            key = variables[key] if key in variables else __builtins__[key]
+                            key = (
+                                variables[key]
+                                if key in variables
+                                else __builtins__[key]
+                            )
                         current = current[key]
                         # is_first = False
                         element_chars = []
@@ -1124,13 +1276,23 @@ def from_name(path, accept_dict_as_object=False, **variables):
                 element = "".join(element_chars)
                 if is_first:
                     if in_var:
-                        current = variables[element] if element in variables else __builtins__[element]
+                        current = (
+                            variables[element]
+                            if element in variables
+                            else __builtins__[element]
+                        )
                     else:
-                        raise Exception("firts element of path must be a name_of_variable")
+                        raise Exception(
+                            "firts element of path must be a name_of_variable"
+                        )
                     is_first = False
                 else:
                     if in_var:
-                        element = variables[element] if element in variables else __builtins__[element]
+                        element = (
+                            variables[element]
+                            if element in variables
+                            else __builtins__[element]
+                        )
                     current = _getattr(
                         current, element, accept_dict_as_object
                     )  # permet de marcher avec slot et properties,mais pas getters
@@ -1144,13 +1306,23 @@ def from_name(path, accept_dict_as_object=False, **variables):
                 element = "".join(element_chars)
                 if is_first:
                     if in_var:
-                        current = variables[element] if element in variables else __builtins__[element]
+                        current = (
+                            variables[element]
+                            if element in variables
+                            else __builtins__[element]
+                        )
                     else:
-                        raise Exception("firts element of path must be a name_of_variable")
+                        raise Exception(
+                            "firts element of path must be a name_of_variable"
+                        )
                     is_first = False
                 else:
                     if in_var:
-                        element = variables[element] if element in variables else __builtins__[element]
+                        element = (
+                            variables[element]
+                            if element in variables
+                            else __builtins__[element]
+                        )
                     current = _getattr(
                         current, element, accept_dict_as_object
                     )  # permet de marcher avec slot et properties,mais pas getters
@@ -1167,12 +1339,20 @@ def from_name(path, accept_dict_as_object=False, **variables):
         element = "".join(element_chars)
         if is_first:
             if in_var:
-                current = variables[element] if element in variables else __builtins__[element]
+                current = (
+                    variables[element]
+                    if element in variables
+                    else __builtins__[element]
+                )
             else:
                 raise Exception("firts element of path must be a name_of_variable")
         else:
             if in_var:
-                element = variables[element] if element in variables else __builtins__[element]
+                element = (
+                    variables[element]
+                    if element in variables
+                    else __builtins__[element]
+                )
             current = _getattr(
                 current, element, accept_dict_as_object
             )  # permet de marcher avec slot et properties,mais pas getters
@@ -1184,9 +1364,14 @@ def _getattr(obj, attribut, accept_dict_as_object):
         return obj[attribut]
     else:
         try:
-            return getattr(obj, attribut)  # permet de marcher avec slot et properties,mais pas getters
+            # permet de marcher avec slot et properties,mais pas getters
+            return getattr(obj, attribut)
         except:
-            for methode in [f"get{attribut}", f"get_{attribut}", f"get{attribut[0].upper()}{attribut[1:]}"]:
+            for methode in [
+                f"get{attribut}",
+                f"get_{attribut}",
+                f"get{attribut[0].upper()}{attribut[1:]}",
+            ]:
                 if hasattr(obj, methode):
                     return getattr(obj, methode)()
             raise
@@ -1205,7 +1390,8 @@ def _get_getters(extra_getters):
                 getters_[class_] = True
         elif extra_getters is not None:
             raise TypeError(
-                "Encoder getters argument must be None, bool, list, tuple, set or dict, not '%s'" % type(extra_getters)
+                "Encoder getters argument must be None, bool, list, tuple, set or dict, not '%s'"
+                % type(extra_getters)
             )
         return getters_
 
@@ -1225,7 +1411,8 @@ def _get_setters(extra_setters):
                 setters_[class_] = True
         else:
             raise TypeError(
-                "Decoder setters argument must be None, bool, list, tuple, set or dict, not '%s'" % type(extra_setters)
+                "Decoder setters argument must be None, bool, list, tuple, set or dict, not '%s'"
+                % type(extra_setters)
             )
         return setters_
 
@@ -1252,5 +1439,4 @@ def _get_properties(extra_properties):
 
 
 # --- Import of plugins -------------------------------------------------------
-
-from . import plugins  # needed for plugin regord
+from . import plugins  # le laisser à la fin ! needed for plugin regord
